package com.ims.eval.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationDeptBusinessAssessment;
import com.ims.eval.entity.EvaluationDeptBusinessContent;
import com.ims.eval.entity.custom.Path;
import com.ims.eval.entity.dto.request.DeptDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationDeptBusinessAssessmentService;
import com.ims.eval.service.IEvaluationDeptBusinessContentService;
import com.ims.eval.util.ExcelUtil;
import com.ims.eval.util.WordUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 本部部门业绩指标考核
 *
 * @author hlf
 * @date 2023/4/28 10:09
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-dept-assessment")
public class DepartmentalPerformanceIndicatorAssessmentController {

	@Autowired
	private IEvaluationDeptBusinessAssessmentService evaluationDeptBusinessAssessmentService;

	@Autowired
	private IEvaluationDeptBusinessContentService evaluationDeptBusinessContentService;

	/**
	 * 部门考评业务列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param responsibilityCode 业务编码
	 * @param des                业务简述
	 * @param appraisalYear      月度
	 * @param stage              流程状态
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "deptId") String deptId,
		@RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
		@RequestParam(value = "des", required = false) String des,
		@RequestParam(value = "appraisalYear", required = false) String appraisalYear,
		@RequestParam(value = "stage", required = false) String stage) {
		IPage<EvaluationDeptBusinessAssessment> list = evaluationDeptBusinessAssessmentService.listPage(pageNum, pageSize, deptId, responsibilityCode, des, appraisalYear, stage);
		return R.ok().data(list);
	}

	/**
	 * 部门考评业务信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationDeptBusinessAssessment> list = evaluationDeptBusinessAssessmentService.list();
		return R.ok().data(list);
	}

	/**
	 * 生成报表
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/generateReport/{id}")
	public R generateReport(@PathVariable("id") String id) throws Exception {
		Map<String, Object> dataMap = new HashMap<>();
		EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = evaluationDeptBusinessAssessmentService.getById(id);
		String[] deptIds = evaluationDeptBusinessAssessment.getDeptId().split(",");
		String[] deptNames = evaluationDeptBusinessAssessment.getDeptName().split(",");
		List<DeptDTO> deptList = new ArrayList<>();
		for (int i = 0; i < deptIds.length; i++) {
			DeptDTO dept = new DeptDTO();
			dept.setDeptId(deptIds[i]);
			dept.setDeptName(deptNames[i]);
			List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList = evaluationDeptBusinessContentService.selectEvaluationDeptBusinessContentListByBusinessPlanId(id, deptIds[i]);
			dept.setList(evaluationDeptBusinessContentList);
			deptList.add(dept);
		}
		dataMap.put("appraisalYear", evaluationDeptBusinessAssessment.getAppraisalYear());
		dataMap.put("list", deptList);
		String fileName = evaluationDeptBusinessAssessment.getAppraisalYear() + "年度经营业绩完成情况.doc";
		WordUtil.generateWord(dataMap, "template_dbkp_wcqk.ftl", Path.getStatementPath(), fileName);
		return R.ok().data(fileName);
	}

	/**
	 * 下载报表
	 *
	 * @param request    request
	 * @param response   response
	 * @param reportName 文件名
	 * @throws IOException 异常
	 */
	@GetMapping(value = "/downloadReport/{reportName}")
	public void downloadReport(HttpServletRequest request, HttpServletResponse response, @PathVariable("reportName") String reportName) throws IOException {
		//设置文件路径
		File file = new File(Path.getStatementPath() + reportName);
		//判断文件是否存在
		if (file.exists()) {
			//设置强制下载不打开
			response.setContentType("application/force-download");
			//设置文件名
			response.addHeader("Content-Disposition", "attachment;fileName=".concat(String.valueOf(URLEncoder.encode(reportName, "UTF-8"))));
			byte[] buffer = new byte[1024];
			FileInputStream fis = null;
			BufferedInputStream bis = null;
			try {
				fis = new FileInputStream(file);
				bis = new BufferedInputStream(fis);
				OutputStream os = response.getOutputStream();
				int i = bis.read(buffer);
				while (i != -1) {
					os.write(buffer, 0, i);
					i = bis.read(buffer);
				}
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				if (bis != null) {
					try {
						bis.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				if (fis != null) {
					try {
						fis.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	/**
	 * 修改状态
	 *
	 * @param id    主键
	 * @param stage 状态
	 * @return 结果
	 */
	@PostMapping(value = "/targetStart")
	public R targetStart(@RequestParam(value = "id") String id,
						 @RequestParam(value = "stage") String stage) {
		try {
			EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = evaluationDeptBusinessAssessmentService.getById(id);
			evaluationDeptBusinessAssessment.setStage(stage);
			boolean b = evaluationDeptBusinessAssessmentService.updateById(evaluationDeptBusinessAssessment);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.customError("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-头部信息
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	@GetMapping(value = "/detailsHead/{id}")
	public R detailsHead(@PathVariable("id") String id) {
		EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = evaluationDeptBusinessAssessmentService.getById(id);
		return R.ok().data(evaluationDeptBusinessAssessment);
	}

	/**
	 * 校验审核状态
	 *
	 * @param id 主键
	 * @return 结果
	 */
	@GetMapping(value = "/checkAuditStatus/{id}/{deptId}")
	public R checkAuditStatus(@PathVariable String id, @PathVariable String deptId) {
		boolean b = true;
		List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList = evaluationDeptBusinessContentService.selectEvaluationDeptBusinessContentListByBusinessPlanId(id, deptId);
		for (EvaluationDeptBusinessContent evaluationDeptBusinessContent : evaluationDeptBusinessContentList) {
			if (!"0".equals(evaluationDeptBusinessContent.getAuditStatusKp())) {
				b = false;
				break;
			}
		}
		if (b) {
			return R.ok().data(b);
		} else {
			return R.customError("行数据审核状态未通过！");
		}
	}

	/**
	 * 导入
	 *
	 * @param file 文件
	 * @return 结果
	 */
	@PostMapping(value = "/import")
	public R importData(@RequestParam("file") MultipartFile file, @RequestParam("id") String id) {
		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					EvaluationDeptBusinessContent evaluationDeptBusinessContent = new EvaluationDeptBusinessContent();
					evaluationDeptBusinessContent.setBusinessPlanId(id);
					evaluationDeptBusinessContent.setDeptId(str[0]);
					evaluationDeptBusinessContent.setDeptName(str[1]);
					evaluationDeptBusinessContent.setSerialNumber(Integer.valueOf(str[2]));
					evaluationDeptBusinessContent.setTargetName(str[3]);
					evaluationDeptBusinessContent.setTargetValue(str[4]);
					evaluationDeptBusinessContent.setEvaluationCriteria(str[5]);
					evaluationDeptBusinessContent.setBaseScore(str[6]);
					evaluationDeptBusinessContentList.add(evaluationDeptBusinessContent);
				}
				boolean b = evaluationDeptBusinessContentService.importAssessmentDataList(evaluationDeptBusinessContentList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.customError("导入失败！");
				}
			} catch (Exception e) {
				log.error("错误", e);
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.customError("上传文件为空！");
	}
}
